@@ -58,10 +58,10 @@ test_that("a single instance of missing data won't induce a new genotype", {
 test_that("mlg.filter will reduce the number of multilocus lineages", {
   skip_on_cran()
   mlg.filter(sc) <- 0.4
-  expect_equal(nmll(sc), 24)
+  expect_lt(nmll(sc), nmll(sc, "original"))
 })
 
 test_that("mlg.filter<- can't be used for genlight objects", {
   skip_on_cran()
   expect_warning(mlg.filter(gl) <- 0.5, "mlg.filter<- only has an effect on snpclone objects.")
-})
\ No newline at end of file
+})
